
package eu.dnetlib.dhp.patchrefereed;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.dnetlib.dhp.schema.oaf.KeyValue;
import eu.dnetlib.dhp.schema.oaf.Publication;
import jdk.nashorn.internal.ir.annotations.Ignore;

public class PatchRefereedTest {

	private static final Logger log = LoggerFactory.getLogger(PatchRefereedTest.class);

	private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

	private static SparkSession spark;

	private static Path workingDir;

	@BeforeAll
	public static void beforeAll() throws IOException {
		workingDir = Files.createTempDirectory(PatchRefereedTest.class.getSimpleName());
		log.info("using work dir {}", workingDir);

		SparkConf conf = new SparkConf();
		conf.setAppName(PatchRefereedTest.class.getSimpleName());

		conf.setMaster("local[*]");
		conf.set("spark.driver.host", "localhost");
		conf.set("hive.metastore.local", "true");
		conf.set("spark.ui.enabled", "false");
		conf.set("spark.sql.warehouse.dir", workingDir.toString());
		conf.set("hive.metastore.warehouse.dir", workingDir.resolve("warehouse").toString());

		spark = SparkSession
			.builder()
			.appName(PatchRefereedTest.class.getSimpleName())
			.config(conf)
			.getOrCreate();
	}

	@AfterAll
	public static void afterAll() throws IOException {
		FileUtils.deleteDirectory(workingDir.toFile());
		spark.stop();
	}

	@Test
	public void test2() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/preparedInfo3.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(1, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(1, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'Unknown'").count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classid = '0000'").count());

	}

	@Test
	public void test3() throws Exception {
		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/preparedInfo2.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(1, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(1, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'Unknown'").count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classid = '0000'").count());

	}

	@Test
	public void test4() throws Exception {
		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/preparedInfo3.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(1, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(1, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'Unknown'").count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classid = '0000'").count());

	}

	@Test
	public void test5() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/prod_publication2.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/preparedInfo4.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(1, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(1, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'PeerReviewd'").count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classid = '0001'").count());

	}

	@Test
	public void test6() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/prod_publication1.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/simpleTest/preparedInfo4.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(1, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(1, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'Unknown'").count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classid = '0000'").count());

	}

	@Test
	public void test8() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass()
						.getResource("/eu/dnetlib/dhp/patchrefereed/test/noMatchUrlLen/prod_publication.json")
						.getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass()
						.getResource("/eu/dnetlib/dhp/patchrefereed/test/noMatchUrlLen/preparedInfo.json")
						.getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(3, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(3, tmp1.count());

		Assertions.assertEquals(3, tmp1.filter("refereed.classname = 'Unknown' and refereed.classid ='0000'").count());

	}

	@Test
	public void test7() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match1/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match1/preparedInfo.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(3, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(3, tmp1.count());

		Assertions.assertEquals(1, tmp1.filter("refereed.classname = 'Unknown' and refereed.classid ='0000'").count());

		Assertions
			.assertEquals(1, tmp1.filter("refereed.classname = 'PeerReviewd' and refereed.classid ='0001'").count());

		Assertions
			.assertEquals(1, tmp1.filter("refereed.classname = 'NonPeerReviewd' and refereed.classid ='0002'").count());

	}

	@Test
	public void test9() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match2/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match2/preparedInfo.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(3, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select id, inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(4, tmp1.count());

		Assertions.assertEquals(3, tmp1.filter("refereed.classname = 'Unknown' and refereed.classid ='0000'").count());

		Assertions
			.assertEquals(1, tmp1.filter("refereed.classname = 'PeerReviewd' and refereed.classid ='0001'").count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|orcid_______::6c6960227e2775a019090d8f33e0e6e5' and " +
							"refereed.classname = 'PeerReviewd' and refereed.classid ='0001'")
					.count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|orcid_______::6c6960227e2775a019090d8f33e0e6e5' and " +
							"refereed.classname = 'Unknown' and refereed.classid ='0000'")
					.count());

	}

	@Test
	public void test10() throws Exception {

		SparkPatchRefereed
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match3/prod_publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/publication",
					"-preparedInfoPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/test/match3/preparedInfo.json").getPath()
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<Publication> tmp = sc
			.textFile(workingDir.toString() + "/publication")
			.map(item -> OBJECT_MAPPER.readValue(item, Publication.class));

		Assertions.assertEquals(4, tmp.count());

		org.apache.spark.sql.Dataset<Publication> verificationDataset = spark
			.createDataset(tmp.rdd(), Encoders.bean(Publication.class));

		verificationDataset.createOrReplaceTempView("dataset");

		Dataset<Row> tmp1 = spark.sql("select id, inst.refereed from dataset lateral view explode(instance) i as inst");

		Assertions.assertEquals(4, tmp1.count());

		tmp1.show(false);

		Assertions.assertEquals(2, tmp1.filter("refereed.classname = 'Unknown' and refereed.classid ='0000'").count());

		Assertions
			.assertEquals(
				2, tmp1.filter("refereed.classname = 'nonPeerReviewed' and refereed.classid ='0002'").count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|core_ac_uk__::05d41a3ffee162ae2561befeb718ccf7' and  " +
							"refereed.classname = 'Unknown' and refereed.classid ='0000'  ")
					.count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|core_ac_uk__::b32ca62afc4299e1f43515bc3c33b58f'  and " +
							"refereed.classname = 'Unknown' and refereed.classid ='0000'")
					.count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|core_ac_uk__::127d5c16ad41fe6cf2d196abbae5596e' and  " +
							"refereed.classname = 'nonPeerReviewed' and refereed.classid ='0002'  ")
					.count());

		Assertions
			.assertEquals(
				1, tmp1
					.filter(
						"id = '50|core_ac_uk__::4810ff7dc1a3db8c2568d5fe7245e067'  and " +
							"refereed.classname = 'nonPeerReviewed' and refereed.classid ='0002'")
					.count());

	}

	@Test
	public void remove() {
		KeyValue cf1 = new KeyValue();
		KeyValue cf2 = new KeyValue();

		cf1.setKey("10|openaire____::0a836ef43dcb67bb7cbd4dd509b11b73");
		cf1.setValue("CORE (RIOXX-UK Aggregator)");

		cf2.setKey("10|openaire____::0a836ef43dcb67bb7cbd4dd509b11b73");
		cf2.setValue("CORE (RIOXX-UK Aggregator)");

		System.out.println(cf1.equals(cf2));

		List<String> url1 = Arrays.asList("http://wrap.warwick.ac.uk/66746/1/WRAP_THESIS_Ritchie_2014.pdf");

		List<String> url2 = Arrays.asList("http://wrap.warwick.ac.uk/66746/1/WRAP_THESIS_Ritchie_2014.pdf");

		System.out.println(SparkPatchRefereed.equals(url1, url2));
	}
}
