
package eu.dnetlib.dhp.patchrefereed;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

import org.apache.commons.io.FileUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.SparkSession;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

public class PrepareInfoTest {
	private static final Logger log = LoggerFactory.getLogger(PrepareInfoTest.class);

	private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

	private static SparkSession spark;

	private static Path workingDir;

	@BeforeAll
	public static void beforeAll() throws IOException {
		workingDir = Files.createTempDirectory(PrepareInfoTest.class.getSimpleName());
		log.info("using work dir {}", workingDir);

		SparkConf conf = new SparkConf();
		conf.setAppName(PrepareInfoTest.class.getSimpleName());

		conf.setMaster("local[*]");
		conf.set("spark.driver.host", "localhost");
		conf.set("hive.metastore.local", "true");
		conf.set("spark.ui.enabled", "false");
		conf.set("spark.sql.warehouse.dir", workingDir.toString());
		conf.set("hive.metastore.warehouse.dir", workingDir.resolve("warehouse").toString());

		spark = SparkSession
			.builder()
			.appName(PrepareInfoTest.class.getSimpleName())
			.config(conf)
			.getOrCreate();
	}

	@AfterAll
	public static void afterAll() throws IOException {
		FileUtils.deleteDirectory(workingDir.toFile());
		spark.stop();
	}

	@Test
	public void test1() throws Exception {

		SparkPrepareResultInstanceList
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/prepareinfo/publication.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/resultInstance"
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<ResultInstance> tmp = sc
			.textFile(workingDir.toString() + "/resultInstance")
			.map(item -> OBJECT_MAPPER.readValue(item, ResultInstance.class));

		Assertions.assertEquals(0, tmp.count());

	}

	@Test
	public void test2() throws Exception {
		SparkPrepareResultInstanceList
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/prepareinfo/publication2.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/resultInstance"
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<ResultInstance> tmp = sc
			.textFile(workingDir.toString() + "/resultInstance")
			.map(item -> OBJECT_MAPPER.readValue(item, ResultInstance.class));

		Assertions.assertEquals(0, tmp.count());
	}

	@Test
	public void test3() throws Exception {
		SparkPrepareResultInstanceList
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/prepareinfo/publications.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/resultInstance"
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<ResultInstance> tmp = sc
			.textFile(workingDir.toString() + "/resultInstance")
			.map(item -> OBJECT_MAPPER.readValue(item, ResultInstance.class));

		Assertions.assertEquals(2, tmp.count());
	}

	@Test
	public void test4() throws Exception {
		SparkPrepareResultInstanceList
			.main(
				new String[] {
					"-isSparkSessionManaged", Boolean.FALSE.toString(),
					"-inputPath",
					getClass().getResource("/eu/dnetlib/dhp/patchrefereed/prepareinfo/publication3.json").getPath(),
					"-resultTableName", "eu.dnetlib.dhp.schema.oaf.Publication",
					"-outputPath", workingDir.toString() + "/resultInstance"
				});

		final JavaSparkContext sc = JavaSparkContext.fromSparkContext(spark.sparkContext());

		JavaRDD<ResultInstance> tmp = sc
			.textFile(workingDir.toString() + "/resultInstance")
			.map(item -> OBJECT_MAPPER.readValue(item, ResultInstance.class));

		Assertions.assertEquals(2, tmp.count());
	}

}
